module Bindings (reshape, keyboardMouse) where

import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility
import Graphics.Rendering.OpenGL
import Graphics.UI.GLUT
import Data.IORef

import Matrix

reshape s@(Size w h) = do
  viewport $= (Position 0 0, s)

  glMatrixMode gl_PROJECTION
  loadIdentity

  perspective 45.0 ((fromIntegral w) / (fromIntegral h)) 0.1 2000.0

  glMatrixMode gl_MODELVIEW
  loadIdentity
  

  postRedisplay Nothing

-- zoom out
keyboardAct _ _ p a (Char '-') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y,(-0.1) + (z+0.1)*2)

--zoom in
keyboardAct _ _ p a (Char '+') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y,(-0.1) + (z+0.1)/2)
  
-- move right
keyboardAct _ _ p a (SpecialKey KeyRight) Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x-0.1,y,z)

-- move left
keyboardAct _ _ p a (SpecialKey KeyLeft) Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x+0.1,y,z)

-- move down
keyboardAct _ _ p a (SpecialKey KeyDown) Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y+0.1,z)

-- move up
keyboardAct _ _ p a (SpecialKey KeyUp) Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y-0.1,z)


-- zoom out (char)
keyboardAct _ _ p a (Char 'e') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y,(-1) + (z+1)*2)

-- zoom in (char)
keyboardAct _ _ p a (Char 'q') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y,(-1) + (z+1)/2)

-- move left (char)
keyboardAct _ _ p a (Char 'a') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x-0.1,y,z)

-- move right (char)
keyboardAct _ _ p a (Char 'd') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x+0.1,y,z)

-- move up (char)
keyboardAct _ _ p a (Char 'w') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y+0.1,z)

-- move down (char)
keyboardAct _ _ p a (Char 's') Down (Modifiers Up _ _) _ = do
  (x,y,z) <- get p
  p $= (x,y-0.1,z)



-- rotate neg. about z-axis
keyboardAct _ n' p a (Char '-') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz-5)
   else do return ()

-- rotate pos. about z-axis
keyboardAct _ n' p a (Char '+') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz+5)
   else do return ()

-- rotate neg. about x-axis
keyboardAct _ n' p a (SpecialKey KeyUp) Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx-5,ry,rz)
   else do return ()

-- rotate pos. about x-axis
keyboardAct _ n' p a (SpecialKey KeyDown) Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx+5,ry,rz)
   else do return ()

-- rotate pos. about y-axis
keyboardAct _ n' p a (SpecialKey KeyRight) Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry+5,rz)
   else do return ()

-- rotate neg. about y-axis
keyboardAct _ n' p a (SpecialKey KeyLeft) Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry-5,rz)
   else do return ()


-- rotate neg. about z-axis (char)
keyboardAct _ n' p a (Char 'E') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz-5)
   else do return ()

-- rotate pos. about z-axis (char)
keyboardAct _ n' p a (Char 'Q') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz+5)
   else do return ()
-- rotate neg. about x-axis (char)
keyboardAct _ n' p a (Char 'W') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx-5,ry,rz)
   else do return ()

-- rotate pos. about x-axis (char)
keyboardAct _ n' p a (Char 'S') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx+5,ry,rz)
   else do return ()

-- rotate pos. about y-axis (char)
keyboardAct _ n' p a (Char 'D') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry+5,rz)
   else do return ()

-- rotate neg. about y-axis (char)
keyboardAct _ n' p a (Char 'A') Down (Modifiers Down _ _) _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry-5,rz)
   else do return ()

keyboardAct points n p a (SpecialKey KeyF1) Down _ _ = do
  points $= []
  n $= 0
  p $= (0.0, 0.0, (-2.4::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))
  
keyboardAct _ _ p a (SpecialKey KeyF2) Down _ _ = do
  p $= (0.0, 0.0, (-2.4::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))
    
keyboardAct points n' p a (Char '\r') Down _ _ = do
  pnts <- get points
  n <- get n'
  n' $= if n == 0 then length pnts - 1 else n
  n <- get n'
  let s0@[[x0],[y0]] = (\ (x, y, z) -> [[x],[y]]) $ head pnts
  let s1@[[x1],[y1]] = (\ (x, y, z) -> [[x],[y]]) $ head $ tail pnts
  let s2@[[x2],[y2]] = (\ (x, y, z) -> [[x],[y]]) $ head $ tail $ tail pnts
  let sn@[[xn],[yn]] = (\ (x, y, z) -> [[x],[y]]) $ pnts !! n
                       
--  let s = [ pnts ]

  let v0s = [ [[ xn - x0 ],[ yn - y0 ]] | 
              (( (xn,yn,_), (x0,y0,_) ),i) <- zip (zip (drop n pnts) pnts) [0..], 
              i `mod` n == 0
            ]    -- get all the 'zero' vectors
  --putStr $ "The points: " ++ show (zip (drop n pnts) pnts) ++ " " ++ show n ++ " " ++ "\n"
  --putStr $ "v0s: " ++ show v0s ++ "\n"

  let vs = [ [[ x' - x ],[ y' - y ]] | 
             (( (x',y',_), (x,y,_) ),i) <- zip (zip (drop 1 pnts) pnts) [0..]
           ]     -- get all the vectors
  --putStr $ "vs: " ++ show vs ++ "\n"

{- [[x ],[y ]]  <- v0s, -- get the v0s
             ([[x'],[y']],i) -}

  let mPnts = map (\ (x, y, z) -> [[x],[y]]) pnts
  let pnts' :: [(GLfloat, GLfloat, GLfloat)]
      pnts' = [head pnts]
  points $= pnts'

  

  let j = 0 -- NOT LIKE THIS IN THE GENERAL CASE 

  mapM_ (\j -> do
           let v0@[[_x0],[_y0]] = v0s !! j
           let t0 = divideMatrix (sqrt (_x0*_x0 + _y0*_y0)) [[_x0, _y0],[(-_y0), _x0]] -- rotation matrix
           let ts = [ divideMatrix (sqrt (_x0*_x0 + _y0*_y0)) [[a, -b], [b, a]] | 
                      ([[a],[b]],i) <- zip vs [0..], 
                      i >= (j*n) && i < (j+1) * n ] -- get the j-th sequence of n vectors
           --putStr $ "V0: " ++ show v0 ++ " " ++ show t0 ++ "\n"
                  
           let ps = [ [[x],[y]] | ((x, y, _), i) <- zip pnts [0..], i >= (j*n) && i < (j+1) * n ] -- get the j-th sequence of n points
           pnts' <- get points
           let transPoint@(x0,y0,_) = last pnts'
           mapM_ (\(t,[[vx],[vy]]) -> do
                    let newPoints = map (\[[x],[y]] -> (x+vx,y+vy, 0)) 
                                     [ timesM t (timesM t0 [[x_-x0],[y_-y0]] ) | ([[x_],[y_]], i) <- zip mPnts [0..] , i > (j * n) && i <= (j+1) * n ]
                
                    --putStr $ "\n\n" ++ show t ++ "\n" ++ show (length newPoints) ++ "\n\n\n"
               
                    pnts' <- get points
                    points $= pnts' ++ newPoints

                 ) $ zip ts ps
   ) [0..length pnts `div` n - 1]
  
  pnts' <- get points
  --putStr $ show (length pnts')
{-  let v0@[[x ],[y ]] = [[xn-x0], [yn-y0]] 
  let v1@[[x1'],[y1']] = [[x2-x1], [y2-y1]] -- the first vector
  let t0 = divideMatrix (sqrt (x*x + y*y)) [[x, y],[(-y), x]] -- transformation matrix to rotate the curve
                                                              -- about (0,0,0) in such way that the direction
                                                              -- of the sum vector of the curve is the same as
                                                              -- the direction of Ox+
  let t1 = divideMatrix (sqrt (x*x + y*y) {-sqrt (x1'*x1' + y1'*y1')-} ) [[x1', -y1'],[y1',x1']] -- transformation
                                                              -- matrix to rotate the curve in the direction of v1

  let mPnts''' = map (\ (x, y, z) -> [[x],[y]]) pnts           -- I don't know how to reuse variables in a do statement =]
  let mPnts'' = [ [[x-x0], [y-y0]] | ([[x],[y]],i) <- zip mPnts''' [0..] ]    -- move the curve, so the first point is at (0,0,0)
--  let mPnts' = [ mPnts''' ]
  let mPnts =  map (timesM t1) (map (timesM t0) mPnts'') -- rotates every point
                                                             -- with matrix t0 and t1
--  let mPnts'' = 
  let pnts' = map (\ [[x],[y]] -> (x+x1, y+y1, 0)) mPnts     -- move the curve back in place -}
--  points $= pnts'
  n <- get n'
  putStr $ show (length pnts') ++ "\n"

keyboardAct pnts n p a (MouseButton LeftButton) Down (Modifiers _ ctrl _) pos = do
  n' <- get n
  if n' == 0 then do
      (Size wi hi) <- get windowSize     
      let 
          wd :: GLfloat
          hd :: GLfloat
          dd :: GLfloat

          wd = fromIntegral wi / 2
          hd = fromIntegral hi / 2
          dd = if (wi < hi) then hd else hd

      p'@(x, y, z) <- get p
      let 
          mx :: GLint
          my :: GLint         
          Position mx my = pos         
            
      pnts' <- get pnts
      if ctrl == Up then
        pnts $= pnts' ++ [((((-z)/2.4::GLfloat)/dd*(fromIntegral mx - wd)) - x, z/2.4/dd*(fromIntegral my - hd) - y, 0.0)]
       else
        pnts $= pnts' ++ [((((-z)/2.4::GLfloat)/dd*(fromIntegral (mx - (mx `mod` 10)) - wd)) - x, z/2.4/dd*(fromIntegral (my - (my `mod` 10)) - hd) - y, 0.0)]
      putStr $ show (wi, hi) ++ " " ++ show (((-z)/2.4)/dd*((fromIntegral mx) - wd) - x, z/2.4/dd*((fromIntegral my) - hd) - y) ++ "\n"
   else do
     return()



keyboardAct _ _ _ _ _ _ _ _ = return ()
keyboardMouse points n pos angle key state modifiers position = do
  keyboardAct points n pos angle key state modifiers position

-- Divides a matrix by the given number
divideMatrix :: GLfloat -> Matrix.Matrix -> Matrix.Matrix
divideMatrix d = map (map (/d))
