module Bindings (reshape, keyboardMouse, motion) where

import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility
import Graphics.Rendering.OpenGL
import Graphics.UI.GLUT
import Data.IORef
import Data.Bits

import Matrix
import Flags

reshape s@(Size w h) = do
  viewport $= (Position 0 0, s)

  glMatrixMode gl_PROJECTION
  loadIdentity

  perspective 45.0 ((fromIntegral w) / (fromIntegral h)) 0.1 2000.0

  glMatrixMode gl_MODELVIEW
  loadIdentity
  

  postRedisplay Nothing

motion p param pos = do
  param' <- get param
  if param' .&. flagPos > 0 then do
    p'@(x, y, z) <- get p      
    (Size wi hi) <- get windowSize     
    let 
      wd :: GLfloat
      hd :: GLfloat
      dd :: GLfloat

      wd = fromIntegral wi / 2
      hd = fromIntegral hi / 2
      dd = if (wi < hi) then hd else hd

    let
      mx :: GLint
      my :: GLint         
      Position mx my = pos         
            
    putStr $ show (wi, hi) ++ " " ++ show (((-z)/2.4)/dd*((fromIntegral mx) - wd) - x, z/2.4/dd*((fromIntegral my) - hd) - y) ++ "\n"
   else do return()

keyboardAct _ _ _ _ (Char '\r') Down (Modifiers Down Down Down) _ params _ = do
  p <- get params
  params $= p `xor` flagFrac
        
-- here the magic happens - higher order pseudo-fractal is generated
keyboardAct points n' p a (Char '\r') Down _ _ _ _ = do
  pnts <- get points
  n <- get n'
  n' $= if n == 0 then length pnts - 1 else n
  n <- get n'
                       
  let v0s = [ [[ xn - x0 ],[ yn - y0 ]] | 
              (( (xn,yn,_), (x0,y0,_) ),i) <- zip (zip (drop n pnts) pnts) [0..], 
              i `mod` n == 0
            ]    -- get all the 'zero' vectors

  let vs = [ [[ x' - x ],[ y' - y ]] | 
             (( (x',y',_), (x,y,_) ),i) <- zip (zip (drop 1 pnts) pnts) [0..]
           ]     -- get all the vectors

  let mPnts = map (\ (x, y, z) -> [[x],[y]]) pnts
  let pnts' :: [(GLfloat, GLfloat, GLfloat)]
      pnts' = [head pnts]
  points $= pnts'

  mapM_ (\j -> do
           let v0@[[_x0],[_y0]] = v0s !! j
           let t0 = divideMatrix (sqrt (_x0*_x0 + _y0*_y0)) [[_x0, _y0],[(-_y0), _x0]] -- rotation matrix
           let ts = [ divideMatrix (sqrt (_x0*_x0 + _y0*_y0)) [[a, -b], [b, a]] | 
                      ([[a],[b]],i) <- zip vs [0..], 
                      i >= (j*n) && i < (j+1) * n ] -- get the j-th sequence of n vectors
                  
           let ps = [ [[x],[y]] | ((x, y, _), i) <- zip pnts [0..], i >= (j*n) && i < (j+1) * n ] -- get the j-th sequence of n points
           pnts' <- get points
           let transPoint@(x0,y0,_) = last pnts'
           mapM_ (\(t,[[vx],[vy]]) -> do
                    let newPoints = map (\[[x],[y]] -> (x+vx,y+vy, 0)) 
                                     [ timesM t (timesM t0 [[x_-x0],[y_-y0]] ) | ([[x_],[y_]], i) <- zip mPnts [0..] , i > (j * n) && i <= (j+1) * n ]
                
                    pnts' <- get points
                    points $= pnts' ++ newPoints

                 ) $ zip ts ps
   ) [0..length pnts `div` n - 1]
  
  pnts' <- get points
  n <- get n'
  putStr $ show (length pnts') ++ "\n"

keyboardAct pnts n p a (MouseButton LeftButton) Down (Modifiers _ ctrl _) pos _ _ = do
  n' <- get n
  if n' == 0 then do
      (Size wi hi) <- get windowSize     
      let 
          wd :: GLfloat
          hd :: GLfloat
          dd :: GLfloat

          wd = fromIntegral wi / 2
          hd = fromIntegral hi / 2
          dd = if (wi < hi) then hd else hd

      p'@(x, y, z) <- get p
      let 
          mx :: GLint
          my :: GLint         
          Position mx my = pos         
            
      pnts' <- get pnts
      if ctrl == Up then
        pnts $= pnts' ++ [((((-z)/2.4::GLfloat)/dd*(fromIntegral mx - wd)) - x, z/2.4/dd*(fromIntegral my - hd) - y, 0.0)]
       else
        pnts $= pnts' ++ [((((-z)/2.4::GLfloat)/dd*(fromIntegral (mx - (mx `mod` 10)) - wd)) - x, z/2.4/dd*(fromIntegral (my - (my `mod` 10)) - hd) - y, 0.0)]
      putStr $ show (wi, hi) ++ " " ++ show (((-z)/2.4)/dd*((fromIntegral mx) - wd) - x, z/2.4/dd*((fromIntegral my) - hd) - y) ++ "\n"
   else do
     return()

-- Preset curves
keyboardAct pnts n p a (Char '1') Down _ _ _ _ = do
  pnts' <- newIORef ([(-1,-0.5,0),(0,0.5,0),(1,-0.5,0)]::[(GLfloat, GLfloat, GLfloat)])  -- koch variant
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '2') Down _ _ _ _ = do
  pnts' <- newIORef ([(-1.5,0,0),(-0.5,0,0),(0,sqrt(3)/2,0),(0.5,0,0),(1.5,0,0)]::[(GLfloat, GLfloat, GLfloat)]) -- koch curve
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '3') Down _ _ _ _ = do
  pnts' <- newIORef ([(-0.21333334,-0.14666668, 0), (-5.3333335e-2,0.22666667,0), (0.29333335,1.3333334e-2, 0), (0.40666667,-0.36666667, 0)]::[(GLfloat, GLfloat, GLfloat)])
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '4') Down _ _ _ _ = do
  pnts' <- newIORef ([(-1,-0.5,0),(0,-0.5,0),(0,0.5,0),(1,0.5,0)]::[(GLfloat, GLfloat, GLfloat)])
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '5') Down _ _ _ _ = do
  pnts' <- newIORef ([(-0.32734954,-8.553327e-2, 0), (-0.3695882,0.40021122, 0), (-3.379092e-2,0.5163675, 0), (0.3632524,0.18268216, 0), (0.20485745,-0.20591342, 0)]::[(GLfloat, GLfloat, GLfloat)])
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '6') Down _ _ _ _ = do
  pnts' <- newIORef ([
                       (-2, 0, 0), (-sqrt(2), sqrt(2), 0), (0, 2, 0), (sqrt(2)/2, 1 + sqrt(2)/2, 0), (1, 1, 0), (sqrt (2)/2, 1 - sqrt(2)/2, 0), (0, 0, 0),
                       (-sqrt(2)/2, sqrt(2)/2 - 1, 0), (-1, -1, 0), (-sqrt(2)/2, -sqrt(2)/2 - 1, 0), (0, -2, 0), (sqrt (2), -sqrt (2), 0), (2, 0, 0)
                       ]::[(GLfloat, GLfloat, GLfloat)])
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '7') Down _ _ _ _ = do
  pnts' <- newIORef ([(-1.5,0,0),(-0.5,0,0),(-0.5,1,0),(0.5,1,0),(0.5,0,0),(1.5,0,0)]::[(GLfloat, GLfloat, GLfloat)])
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '8') Down _ _ _ _ = do
  pnts' <- newIORef ([
                       (-2, 0, 0), (-sqrt(2), sqrt(2), 0), (0, 2, 0), (-sqrt(2)/2, 1 + sqrt(2)/2, 0), (-1, 1, 0), (-sqrt (2)/2, 1 - sqrt(2)/2, 0), (0, 0, 0),
                       (sqrt(2)/2, sqrt(2)/2 - 1, 0), (1, -1, 0), (sqrt(2)/2, -sqrt(2)/2 - 1, 0), (0, -2, 0), (sqrt (2), -sqrt (2), 0), (2, 0, 0)
                       ]::[(GLfloat, GLfloat, GLfloat)])
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '9') Down _ _ _ _ = do
  pnts' <- newIORef ([(-2,0,0),(-1,1,0),(0,1.4,0),(1,1,0),(2,0,0)]::[(GLfloat, GLfloat, GLfloat)])  -- koch variant
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

keyboardAct pnts n p a (Char '0') Down _ _ _ _ = do
  pnts' <- newIORef ([(0, 0, 0), (1-sqrt(2)/2 ,sqrt(2)/2 ,0 ), (1, 1, 0),
                      (1+3/4*sqrt(2), (-1)/2+3/4*sqrt(2) ,0),(5/2,-1/2,0),
                      (1/2 + sqrt(2), -1/2-sqrt(2), 0), (1/2, -5/2, 0),
                      (1/2-5/4*sqrt(2), -5/4*sqrt(2), 0), (-2, 0, 0)
                     ]::[(GLfloat, GLfloat, GLfloat)])  -- koch variant
  pnts'' <- get pnts'
  pnts $= pnts''
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))

-- zoom out
keyboardAct _ _ p a (Char '-') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y,(-0.1) + (z+0.1)*1.5)

--zoom in
keyboardAct _ _ p a (Char '+') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y,(-0.1) + (z+0.1)/1.5)
  
-- move right
keyboardAct _ _ p a (SpecialKey KeyRight) Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x-0.1,y,z)

-- move left
keyboardAct _ _ p a (SpecialKey KeyLeft) Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x+0.1,y,z)

-- move down
keyboardAct _ _ p a (SpecialKey KeyDown) Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y+0.1,z)

-- move up
keyboardAct _ _ p a (SpecialKey KeyUp) Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y-0.1,z)


-- zoom out (char)
keyboardAct _ _ p a (Char 'e') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y,(-1) + (z+1)*1.5)

-- zoom in (char)
keyboardAct _ _ p a (Char 'q') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y,(-1) + (z+1)/1.5)

-- move left (char)
keyboardAct _ _ p a (Char 'a') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x-0.1,y,z)

-- move right (char)
keyboardAct _ _ p a (Char 'd') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x+0.1,y,z)

-- move up (char)
keyboardAct _ _ p a (Char 'w') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y+0.1,z)

-- move down (char)
keyboardAct _ _ p a (Char 's') Down (Modifiers Up _ _) _ _ _ = do
  (x,y,z) <- get p
  p $= (x,y-0.1,z)



-- rotate neg. about z-axis
keyboardAct _ n' p a (Char '-') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz-5)
   else do return ()

-- rotate pos. about z-axis
keyboardAct _ n' p a (Char '+') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz+5)
   else do return ()

-- rotate neg. about x-axis
keyboardAct _ n' p a (SpecialKey KeyUp) Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx-5,ry,rz)
   else do return ()

-- rotate pos. about x-axis
keyboardAct _ n' p a (SpecialKey KeyDown) Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx+5,ry,rz)
   else do return ()

-- rotate pos. about y-axis
keyboardAct _ n' p a (SpecialKey KeyRight) Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry+5,rz)
   else do return ()

-- rotate neg. about y-axis
keyboardAct _ n' p a (SpecialKey KeyLeft) Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry-5,rz)
   else do return ()


-- rotate neg. about z-axis (char)
keyboardAct _ n' p a (Char 'E') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz-5)
   else do return ()

-- rotate pos. about z-axis (char)
keyboardAct _ n' p a (Char 'Q') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry,rz+5)
   else do return ()
-- rotate neg. about x-axis (char)
keyboardAct _ n' p a (Char 'W') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx-5,ry,rz)
   else do return ()

-- rotate pos. about x-axis (char)
keyboardAct _ n' p a (Char 'S') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx+5,ry,rz)
   else do return ()

-- rotate pos. about y-axis (char)
keyboardAct _ n' p a (Char 'D') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry+5,rz)
   else do return ()

-- rotate neg. about y-axis (char)
keyboardAct _ n' p a (Char 'A') Down (Modifiers Down _ _) _ _ _ = do
  n <- get n'
  if n > 0 then do
    (rx,ry,rz) <- get a
    a $= (rx,ry-5,rz)
   else do return ()

keyboardAct points n p a (SpecialKey KeyF1) Down _ _ _ _ = do
  points $= []
  n $= 0
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))
  
keyboardAct _ _ p a (SpecialKey KeyF2) Down _ _ _ _ = do
  p $= (0.0, 0.0, (-6.0::GLfloat))
  a $= (0.0, 0.0, (0.0::GLfloat))    

-- switch the flag for the 'special effect'
keyboardAct _ _ _ _ (Char 'f') Down (Modifiers Up _ _) _ params _ = do
  p <- get params
  params $= p `xor` flagFilled
         
-- switch the flag for the curve
keyboardAct _ _ _ _ (Char 'c') Down (Modifiers Up _ _) _ params _ = do
  p <- get params
  params $= p `xor` flagCurve
         
-- switch the flag for the points
keyboardAct _ _ _ _ (Char 'p') Down (Modifiers Up _ _) _ params _ = do
  p <- get params
  params $= p `xor` flagPoints
         
-- switch the flag for the grid
keyboardAct _ _ _ _ (Char 'g') Down (Modifiers Up _ _) _ params _ = do
  p <- get params
  params $= p `xor` flagGrid


keyboardAct pnts n p a (Char 't') Down _ _ param _ = do
  p <- get param
  param $= p `xor` flagPos

-- rotate the colors for the 'special effect'     
keyboardAct _ _ _ _ (Char 'F') Down (Modifiers Down _ _) _ _ colors = do
  cs@[ cp1, cp2, cc1, cc2, cf1, cf2 ] <- get colors
  let cf1' = if cf2 == Color3 1 0 (0::GLfloat) then rotateColor cf1
              else cf1
  let cf2' = rotateColor cf2
  colors $= [ cp1, cp2, cc1, cc2, cf1', cf2' ]
  
-- rotate the colors for the curve
keyboardAct _ _ _ _ (Char 'C') Down (Modifiers Down _ _) _ _ colors = do
  cs@[ cp1, cp2, cc1, cc2, cf1, cf2 ] <- get colors
  let cc1' = if cc2 == Color3 1 0 (0::GLfloat) then rotateColor cc1
              else cc1
  let cc2' = rotateColor cc2
  colors $= [ cp1, cp2, cc1', cc2', cf1, cf2 ]
         
-- rotate the colors for the points
keyboardAct _ _ _ _ (Char 'P') Down (Modifiers Down _ _) _ _ colors = do
  cs@[ cp1, cp2, cc1, cc2, cf1, cf2 ] <- get colors
  let cp1' = if cp2 == Color3 1 0 (0::GLfloat) then rotateColor cp1
              else cp1
  let cp2' = rotateColor cp2
  colors $= [ cp1', cp2', cc1, cc2, cf1, cf2 ]
         
-- rotate the colors for the grid   
keyboardAct _ _ _ _ (Char 'G') Down (Modifiers Down _ _) _ _ colors = do
  return ()

keyboardAct _ _ _ _ key _ mods _ _ _ = putStr $ show key ++ " " ++ show mods ++ "\n"
keyboardMouse points n pos angle params colors key state modifiers position = do
  keyboardAct points n pos angle key state modifiers position params colors

-- Divides a matrix by the given number
divideMatrix :: GLfloat -> Matrix.Matrix -> Matrix.Matrix
divideMatrix d = map (map (/d))

rotateColor :: Color3 GLfloat -> Color3 GLfloat
rotateColor (Color3 1 0 0) = Color3 1 1 0
rotateColor (Color3 1 1 0) = Color3 0 1 0
rotateColor (Color3 0 1 0) = Color3 0 1 1
rotateColor (Color3 0 1 1) = Color3 0 0 1
rotateColor (Color3 0 0 1) = Color3 1 0 1
rotateColor (Color3 1 0 1) = Color3 1 1 1
rotateColor (Color3 1 1 1) = Color3 0 0 0
rotateColor (Color3 0 0 0) = Color3 1 0 0
rotateColor _ = Color3 1 1 1