module Display (display,initializeGL,idle) where

import Graphics.Rendering.OpenGL.Raw.ARB.Compatibility
import Graphics.Rendering.OpenGL.Raw.Core31
import Graphics.Rendering.OpenGL
import Graphics.UI.GLUT
import Data.Bits hiding (rotate)

import Cube
import Flags

phi :: GLfloat
phi = (1 + sqrt (5))/2

display points pos angle n' params' colors = do    
  ( x,   y,  z ) <- get pos
  ( rx, ry, rz ) <- get angle
  p <- get points
  n <- get n'
  params <- get params'
  
  clear [ ColorBuffer, DepthBuffer ]

  loadIdentity
  translate $ Vector3 x y (z::GLfloat)

  rotate rx $ Vector3 1.0 0.0 (0.0::GLfloat)
  rotate ry $ Vector3 0.0 1.0 (0.0::GLfloat)
  rotate rz $ Vector3 0.0 0.0 (1.0::GLfloat)

  --cube (1.0::GLfloat)


{-  renderPrimitive LineStrip $ do
    vertex $ Vertex3   1.0  (-1.0) (0.0::GLfloat)
    vertex $ Vertex3 (-1.0) (-1.0) (0.0::GLfloat)
    vertex $ Vertex3 (-1.0)   1.0  (0.0::GLfloat)
    vertex $ Vertex3   1.0    1.0  (0.0::GLfloat)
    vertex $ Vertex3   1.0  (-1.0) (0.0::GLfloat)
-}    
  
  -- Draw grid
  if params .&. flagFrac > 0 then do

    if params .&. flagGrid > 0 then do
      if length p > 0 then do
        let minx = (minimum $ map (\(x,y,z) -> x) p) - 0.1
        let maxx = (maximum $ map (\(x,y,z) -> x) p) + 0.1
        let miny = (minimum $ map (\(x,y,z) -> y) p) - 0.1               
        let maxy = (maximum $ map (\(x,y,z) -> y) p) + 0.1

        renderPrimitive Lines $ do
          color $ Color3 0.2 0.2 (0.8::GLfloat)
          mapM_ (\x -> do           
                   vertex $ Vertex3 x miny 0
                   vertex $ Vertex3 x maxy 0
           ) $ [minx, minx + (maxx-minx) / (2 ^ (n+1)) ..  maxx]
          mapM_ (\y -> do
                   vertex $ Vertex3 minx y 0
                   vertex $ Vertex3 maxx y 0
           ) $ [miny, miny + (maxy - miny) / (2 ^ (n+1)) ..  maxy]
       else return ()
     else return ()
    
    [ cp1, cp2, cc1, cc2, cf1, cf2 ] <- get colors
  
    if length p > 1 then do
        color $ Color3 1.0 1.0 (1.0::GLfloat)
        if params .&. flagFilled > 0 then do
          renderPrimitive Polygon $ do
            color $ Color3 0 0 (0::GLfloat)
            vertex $ Vertex3 0 0 (-2::GLfloat)
            mapM_ (\((x,y,z),i) -> preservingMatrix $ do
              color $ mix ((i / fromIntegral (length p - 1))::GLfloat) cf1 cf2
              -- color $ Color3 (0.75 - (i / fromIntegral (length p - 1)) / 2) (1.0 - (i / fromIntegral (length p - 1)) / 2) ((0.75 - (i / fromIntegral (length p - 1)) / 2)::GLfloat)
              vertex $ Vertex3 x y z 
             ) $ zip p [0.0,1.0 ..]
         else do return ()
        if params .&. flagCurve > 0 then do                         
          renderPrimitive LineStrip $ do
            mapM_ (\((x,y,z),i) -> preservingMatrix $ do
              color $ mix ((i / fromIntegral (length p - 1))::GLfloat) cc1 cc2 
              -- color $ Color3 (i / fromIntegral (length p - 1)) 1.0 (1.0::GLfloat)
              vertex $ Vertex3 x y z 
             ) $ zip p [0.0,1.0 ..]
         else do return ()
        if params .&. flagPoints > 0 then do                                     
          renderPrimitive Points $ do
            mapM_ (\((x,y,z),i) -> preservingMatrix $ do
              color $ mix ((i / fromIntegral (length p - 1))::GLfloat) cp1 cp2
              vertex $ Vertex3 x y z 
             ) $ zip p [0.0,1.0 ..] 
         else do return ()
        
     else do
        color $ Color3 1.0 1.0 (1.0::GLfloat)
        renderPrimitive Points $ do
          mapM_ (\(x,y,z) -> preservingMatrix $ do
            vertex $ Vertex3 x y z 
           ) $ p
   else do
     color $ Color3 0.2 0.8 (0.4::GLfloat)
     renderPrimitive LineLoop $ do 
          vertex $ Vertex3 0.0 0.0 (0.0::GLfloat)
          vertex $ Vertex3 (phi*phi) 0.0 (0.0::GLfloat)
          vertex $ Vertex3 (phi*phi) 1.0 (0.0::GLfloat)
          vertex $ Vertex3 0.0 1.0 (0.0::GLfloat)

     renderPrimitive LineLoop $ do 
          vertex $ Vertex3 2.0 0.0 (0::GLfloat)
          vertex $ Vertex3 2.0 0.0 (-phi::GLfloat)
          vertex $ Vertex3 2.0 1.0 (-phi::GLfloat)
          vertex $ Vertex3 2.0 1.0 (0::GLfloat)

     renderPrimitive LineLoop $ do 
          vertex $ Vertex3 0.0 0.0 (-phi::GLfloat)
          vertex $ Vertex3 (phi*phi) 0.0 (-phi::GLfloat)
          vertex $ Vertex3 (phi*phi) 1.0 (-phi::GLfloat)
          vertex $ Vertex3 0.0 1.0 (-phi::GLfloat)

     renderPrimitive LineLoop $ do 
          vertex $ Vertex3 0.0 0.0 (-phi::GLfloat)
          vertex $ Vertex3 (phi*phi) 0.0 (-phi::GLfloat)
          vertex $ Vertex3 (phi*phi) 1.0 (-phi::GLfloat)
          vertex $ Vertex3 0.0 1.0 (-phi::GLfloat)

     renderPrimitive LineLoop $ do 
          vertex $ Vertex3 2.0 0.0 (-phi+2*phi*phi-4::GLfloat)
          vertex $ Vertex3 2.0 1.0 (-phi+2*phi*phi-4::GLfloat)
          vertex $ Vertex3 (phi*phi) 1.0 (-phi+2*phi*phi-4::GLfloat)
          vertex $ Vertex3 (phi*phi) 0.0 (-phi+2*phi*phi-4::GLfloat)

     renderPrimitive Lines $ do
          vertex $ Vertex3 0 0 (0::GLfloat)
          vertex $ Vertex3 0 0 (-phi::GLfloat)
          vertex $ Vertex3 (phi*phi) 0.0 (0::GLfloat)
          vertex $ Vertex3 (phi*phi) 0.0 (-phi::GLfloat)
          vertex $ Vertex3 (phi*phi) 1.0 (0::GLfloat)
          vertex $ Vertex3 (phi*phi) 1.0 (-phi::GLfloat)
          vertex $ Vertex3 0.0 1.0 (0::GLfloat)
          vertex $ Vertex3 0.0 1.0 (-phi::GLfloat)
          
    
          
  
  swapBuffers
    
initializeGL = do
  
  shadeModel $= Smooth
  clearDepth $= (1.0::GLclampd)
  glEnable gl_DEPTH_TEST
  depthFunc $= Just Lequal
  glHint gl_PERSPECTIVE_CORRECTION_HINT gl_NICEST
 
idle = do 
  postRedisplay Nothing

mix :: GLfloat -> Color3 GLfloat -> Color3 GLfloat -> Color3 GLfloat
mix p (Color3 r1 g1 b1) (Color3 r2 g2 b2) = Color3 (r1 * (1-p) + r2 * p) (g1 * (1-p) + g2 * p) (b1 * (1-p) + b2 * p)