{-
  Author: Stanislav Manilov
  e-mail: stanislav.manilov@gmail.com
  Program: Fractal Illustrator

  This small program has the aim to illustrate fractals that consist of segments connected
  in a curve and that can be build by changing each segment of the previous order fractal
  by the initial curve. An example for such fractal is the Koch curve. 

  The program gives the ability to move and rotate the result in the three dimensions for 
  investigating it from different angles. It also can show a grid on the drawing plane,
  display separately or together the points of the curve, the curve and one 'special effect'.


  F1 - resets points
  F2 - resets view

  left mouse button - plots a point              [ while in plotting mode ]
  ctrl + left mouse button - plots a point on a 10x10 pixel grid
  enter - recurses the next step of the fractal  [ enters fractal mode if in plotting mode ]
  
  left arrow  / a - moves scene to the left
  right arrow / d - moves scene to the right
  up arrow    / w - moves scene up
  down arrow  / s - moves scene down
  +           / e - zooms into the scene
  -           / q - zooms out of the scene

  g - shows/hides grid
  p - shows/hides points
  c - shows/hides curve
  f - shows/hides 'special effect'
  t - shows mouse position in console

  shift + p - changes the color of the points
  shift + c - changes the color of the curve
  shift + f - changes the color of the 'special effect'

  shift + left arrow  / shift + a - rotates positively the scene about the y - axis  [ aviable in fractal mode ]
  shift + right arrow / shift + d - rotates negatively the scene about the y - axis  [ aviable in fractal mode ]
  shift + up arrow    / shift + w - rotates positively the scene about the y - axis  [ aviable in fractal mode ]
  shift + down arrow  / shift + s - rotates negatively the scene about the y - axis  [ aviable in fractal mode ]
  shift + +           / shift + e - rotates positively the scene about the y - axis  [ aviable in fractal mode ]
  shift + -           / shift + q - rotates negatively the scene about the y - axis  [ aviable in fractal mode ]

  If you said to yourself "wow" at anytime using this app (I did !)
  please let me know on stanislav.manilov@gmail.com
-}
module Main where

import Graphics.Rendering.OpenGL
import Graphics.UI.GLUT
import Data.IORef
import Data.Bits

import Bindings
import Flags
import Display

main = do
  (progname,_) <- getArgsAndInitialize  

  initialDisplayMode $= [ DoubleBuffered , WithDepthBuffer]

  createWindow ("Fractal Illustrator")

  initializeGL

  n <- newIORef 0
  pos <- newIORef ( 0.0, 0.0, (-6.0::GLfloat) )
  angle <- newIORef ( 0.0, 0.0, (0.0::GLfloat) )
  params <- newIORef $ flagPoints .|. flagCurve .|. flagFrac -- parameters for displaying
  colors <- newIORef ( [ Color3 1.0 0.0 0.0, Color3 1.0 0.0 0.0, -- the first two colors are for the points
                         Color3 1.0 1.0 1.0, Color3 1.0 1.0 1.0, -- the second two colors are for the curve
                         Color3 0.0 0.2 0.8, Color3 0.0 0.2 0.8 -- the third two colors are for the polygons
                       ]::[Color3 GLfloat] )


  points <- newIORef ([]::[(GLfloat, GLfloat, GLfloat)])

  keyboardMouseCallback $= Just (keyboardMouse points n pos angle params colors)
  passiveMotionCallback $= Just (motion pos params)

  reshapeCallback $= Just reshape
  displayCallback $= display points pos angle n params colors

  idleCallback $= Just idle
  
  mainLoop