module Matrix where

import Data.Char
import Data.List
import Data.Ratio
import Graphics.UI.GLUT

type Matrix = [[GLfloat]]

eqM :: Matrix.Matrix -> Matrix.Matrix -> IO()
eqM m1 m2 = do
  let m12 = [[ a+b |(a,b) <- row]|row <- [zip left right |(left, right) <- (zip m1 m2)]]
  return ()
  

uniform :: [Double] -> Bool
uniform (x:xs) = all (== x) xs

valid :: Matrix.Matrix -> Bool
valid [] = False
valid m3x = Matrix.uniform (map (fromIntegral . length) m3x)

size :: Matrix.Matrix -> (Int,Int)
size [] = (0, 0)
size [line] = (length line, 1) 
size (line:matrix) = (length line, (length matrix) + 1)

square :: Matrix.Matrix -> Bool
square m3x = uncurry (==) (size m3x)

-- 10.
{- 
 Inverse: 
  I  . Indexes the matrix
  II . For every row:
   1  . For every column:
    a  . Remove this row and column from the 
         matrix and evaluate the determinant
         of the result
   2  . Form a row from the determinants, while
        taking the negative value of the ones with
        odd sum of indecesa == b
    where 
      (a, b) = size mx
  III. Form a matrix from the formed rows
  IV . Divide the matrix by the input matrix's 
       determinant
  V  . Transpose itt
 Needs: map
-}
inverse :: Matrix.Matrix -> Matrix.Matrix
inverse stan 
    | not (valid stan) 			= error ("Not a matrix")
    | determinate stan == 0 	= error ("The matrix is singular")
	| not (square stan)			= error ("Matrix not invertable")
	
	| size stan == (1, 1) =
		[[1 / head (head stan)]]
    | otherwise = 
        divideMatrix (determinate stan) (map (\(i, r) -> getDeterminants i (transpose stan)) indexedStan)
            where
				-- Indexes the input matrix
				-- Needs: zip
				indexedStan :: [(Int, [GLfloat])]
				indexedStan = zip [0..length(stan)] stan
								
				-- Remove the row with the given index from the 
				-- matrix and give all 'vertical' determinants 
				-- Needs: map, determinate, reduceMatrix
				getDeterminants :: Int -> Matrix.Matrix-> [GLfloat]
				getDeterminants rowIndex matrix = map (\i -> (-1)^(i+rowIndex) * determinate (reduceMatrix rowIndex i matrix)) [0..length (head matrix)-1]
				
				-- Determinates a matrix
				-- Needs: transpose, reduceMatrix
				determinate :: Matrix.Matrix -> GLfloat
				determinate m3x 
					| null m3x 				=
					  error ("Empty matrix")
					| not (square m3x) 		= 
					  error ("Not a square matrix")				
					| size m3x == (1, 1) 	= 
					   head (head m3x)
					| size m3x == (2, 2) 	= 
					  ((m3x !! 0) !! 0) * ((m3x !! 1) !! 1) -
					  ((m3x !! 0) !! 1) * ((m3x !! 1) !! 0)
					| otherwise =
						sum (map getSubDeterminant numberedFirstRow)
						where 
							numberedFirstRow = (head m3x) `zip` [0..fst(size m3x)-1]
							getSubDeterminant (k, n) = (-1)^n * k * (determinate (reduceMatrix 0 n m3x))					


                                -- Divides a matrix by the given number
                                divideMatrix :: GLfloat -> Matrix.Matrix -> Matrix.Matrix
                                divideMatrix d = map (map (/d))

				-- Removes the i-th row and j-th column from
				-- a matrix
				-- Needs: map, removeEl
				reduceMatrix :: Int -> Int -> Matrix.Matrix -> Matrix.Matrix
				reduceMatrix i j mx = map (removeEl j) (removeEl i mx)

				-- Removes the i-th element of a list
				-- Needs: take, drop
				removeEl :: Int -> [a] -> [a]
				removeEl i list = (take i list) ++ (drop (i+1) list)		
							
							
-- Identity matrices of given size
--
idM :: Int -> Matrix.Matrix
idM n | 0 < n = map f [0..n-1]
 where
   f m = take n (replicate m 0 ++ [1] ++ repeat 0)

timesM :: Matrix.Matrix -> Matrix.Matrix -> Matrix.Matrix
timesM m1 m2 
    |not (valid m1) || not (valid m2) =
        error ("Invalid matrix(ces)")
    |fst (size m1) /= snd (size m2) =
        error ("Matrices are of incorrect size")
    |otherwise = 
        [[innerProduct i j| j <- m2']| i <- m1]      
    where
      m2' = transpose m2
      innerProduct :: [GLfloat] -> [GLfloat] -> GLfloat
      innerProduct xs ys = sum (zipWith (*) xs ys)

      multRow :: [GLfloat] -> Matrix.Matrix -> [GLfloat]
      multRow xs mx = map (innerProduct xs) mx


			
			
			
			